@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
     pygments.lexers.automation
     ~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -327,6 +326,7 @@ class AutoItLexer(RegexLexer):
             include('builtInFunctions'),
             include('builtInMarcros'),
             (r'"', String, combined('stringescape', 'dqs')),
+            (r"'", String, 'sqs'),
             include('numbers'),
             (r'[a-zA-Z_#@$][\w#@$]*', Name),
             (r'\\|\'', Text),
@@ -368,6 +368,11 @@ class AutoItLexer(RegexLexer):
             (r'"', String, '#pop'),
             include('strings')
         ],
+        'sqs': [
+            (r'\'\'|\`([,%`abfnrtv])', String.Escape),
+            (r"'", String, '#pop'),
+            (r"[^'\n]+", String)
+        ],
         'garbage': [
             (r'[^\S\n]', Text),
         ],
